/**************************************************************************
 * AUTHOR AND LICENCING:
 *	Written by Antony Dzeryn
 *		http://ant.simianzombie.com
 *		spam_mail250@yahoo.com
 *	Copyright 2005 Antony Dzeryn
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT
 *	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc., 675
 *	Mass Ave, Cambridge, MA 02139, USA.
 **************************************************************************/

/**************************************************************************
 * List of functions:
 *  
 * - CopyOptions(sourceId, destId) - copy selected items from source to dest
 * - MoveOptions(sourceId, destId) - move selected items from source to dest
 * - MoveAllOptions(sourceId, destId) - moves all options from source to dest
 * - RemoveOptions(selectId) - remove selected items from select list
 * - RemoveOption(selectId, index) - remove a single option from select list
 * - SelectToInput(selectId, destId, delimiter) - concatenate an entire select
 *    list into an input box
 * - SelectSelectedToInput(selectId, destId, delimiter) - concatenate selected
 *    options within a select element into an input box
 * - MoveUp(selectId) - move selected options in the list up one place
 * - MoveDown(selectId) - move selected options in the list down one place
 * - SwapOption(selectId, sourceIndex, destIndex) - swap two options in the list
 * - SortOptions(selectId) - sorts the options
 * - SelectAllOptions(selectId) - selects every option in the list
 * - SelectNoOptions(selectId) - unselects all options
 **************************************************************************/

/*
 * Copy selected items from source list to destination list
 * param: sourceId (select element id) - source list
 * param: destId (select element id) - destination list
 */   
function CopyOptions(sourceId, destId) {
	// Get objects from parameters
	var sourceObj = document.getElementById(sourceId);
	var destObj = document.getElementById(destId);

	// Loop through all selected items, copying them from source to dest	
	for (var i = 0; i < sourceObj.options.length; i++) {
		if (sourceObj.options[i].selected) {
			// Found a selected item

			// Create new option element and set parameters
			var newElem = document.createElement("OPTION");
			newElem.text = sourceObj.options[i].text;
			newElem.value = sourceObj.options[i].value;

			// Append the new element to destination object
			destObj.options.add(newElem);
		}
	}
}

/*
 * Move all options from source list to destination list
 * param: sourceId (select element id) - source list
 * param: destId (select element id) - destination list
 */   
function MoveAllOptions(sourceId, destId) {
	SelectAllOptions(sourceId);
	MoveOptions(sourceId, destId);
}

/*
 * Move selected options from source list to destination list
 * param: sourceId (select element id) - source list
 * param: destId (select element id) - destination list
 */   
function MoveOptions(sourceId, destId) {
	// Insert new options into dest
	CopyOptions(sourceId, destId);
	// Remove old options from source
	RemoveOptions(sourceId);
}

/*
 * Remove selected options from list
 * param: selectId (select element id) - source list
 */   
function RemoveOptions(selectId) {
	// Get object from ID
	var selectObj = document.getElementById(selectId);
	var i = 0;

	// Check list type (multiple or single select)
	if (selectObj.multiple) {
		// Remove from multiple select list
		while (i < (selectObj.options.length)) {
			if (selectObj.options[i].selected) {
				// remove the item
				RemoveOption(selectId, i);
			} else {
				i++;
			}
		}
	} else {
		// Remove single option from list
		if(selectObj.options.selectedIndex > -1){
			// Remove one item fron single select list
			i = selectObj.options.selectedIndex;
			RemoveOption(selectId, i);
		}
	}
}

/*
 * Remove a single option from the select list
 * param: selectId (select element id) - source list
 * param: index (integer) - index of the option to remove
 */   
function RemoveOption(selectId, index) {
	// Get object from ID
	var selectObj = document.getElementById(selectId);

	// Shift all options up from index downwards
	for (var i = index; i < (selectObj.options.length - 1); i++) {
		selectObj.options[i].text = selectObj.options[i+1].text;
		selectObj.options[i].value = selectObj.options[i+1].value;
		selectObj.options[i].selected = selectObj.options[i+1].selected;
	}
	
	// Remove last element in the list
	selectObj.options[selectObj.options.length - 1] = null;
}

/*
 * Copies all of the select list's option values into a delimited list
 * param: selectId (select element id) - source list
 * param: destId (input element id) - input box to hold string
 * param: delimiter (string) - delimiter to use between values  
 */   
function SelectToInput(selectId, destId, delimiter) {
	// Get objects
	var selectObj = document.getElementById(selectId);
	var destObj = document.getElementById(destId);
	
	// String to hold output
	var output = "";
	
	// Build delimited string
	for (var i = 0; i < selectObj.options.length; i++) {
		output += selectObj.options[i].value;
		output += delimiter;
	}
	
	// Trim final delimiter from string
	output = output.substr(0, output.length - delimiter.length);
	
	// Set input box value
	destObj.value = output;
}

/*
 * Copies select list's selected option values into a delimited list
 * param: selectId (select element id) - source list
 * param: destId (input element id) - input box to hold string
 * param: delimiter (string) - delimiter to use between values  
 */   
function SelectSelectedToInput(selectId, destId, delimiter) {
	// Get objects
	var selectObj = document.getElementById(selectId);
	var destObj = document.getElementById(destId);
	
	// String to hold output
	var output = "";
	
	// Build delimited string
	for (var i = 0; i < selectObj.options.length; i++) {
		if (selectObj.options[i].selected) {
			output += selectObj.options[i].value;
			output += delimiter;
		}
	}
	
	// Trim final delimiter from string
	output = output.substr(0, output.length - delimiter.length);
	
	// Set input box value
	destObj.value = output;
}


/*
 * Moves all selected items in the list up one place
 * param: selectId (select element id) - source list
 */ 
function MoveUp(selectId){
	// Get the object
	var selectObj = document.getElementById(selectId);
	
	// Check if select object supports multiple selection
	if (selectObj.multiple) {
		// Handle multple selection object
		for (var i = 1; i < selectObj.options.length; i++) {
			if ((selectObj.options[i].selected) && (!selectObj.options[i-1].selected)) {
				SwapOption(selectId, i, i - 1);
			}
		}
	} else {
		// Handle single selection object
		var tmpSel = selectObj.selectedIndex;
		
		if ((tmpSel !==0) && (tmpSel !==-1)){
			SwapOption(selectId, tmpSel, tmpSel - 1);
		}
	}
}

/*
 * Moves all selected items in the list down one place
 * param: selectId (select element id) - source list
 */  
function MoveDown(selectId){
	// Get the object
	var selectObj = document.getElementById(selectId);
	
	// Check if select object supports multiple selection
	if (selectObj.multiple) {
		// Handle multple selection object
		for (var i = selectObj.options.length - 2; i >= 0;  i--) {
			if ((selectObj.options[i].selected) && (!selectObj.options[i+1].selected)) {
				SwapOption(selectId, i, i + 1);
			}
		}
	} else {
		// Handle single selection object
		var tmpSel = selectObj.selectedIndex;
		
		if ((tmpSel < selectObj.options.length - 1) && (tmpSel > -1)) {
			SwapOption(selectId, tmpSel, tmpSel + 1);
		}
	}	
}

/*
 * Swaps two options
 * param: selectId (select element id) - source list
 * param: sourceIndex (select index number) - option to swap #1
 * param: destIndex (select index number) - option to swap #2
 */   
function SwapOption(selectId, sourceIndex, destIndex) {
	var selectObj = document.getElementById(selectId);

	var tempText = selectObj.options[destIndex].text;
	var tempValue = selectObj.options[destIndex].value;
	var tempSelect = selectObj.options[destIndex].selected;
	
	selectObj.options[destIndex].text = selectObj.options[sourceIndex].text;
	selectObj.options[destIndex].value = selectObj.options[sourceIndex].value;	
	selectObj.options[destIndex].selected = selectObj.options[sourceIndex].selected;	
	
	selectObj.options[sourceIndex].text = tempText;
	selectObj.options[sourceIndex].value = tempValue;
	selectObj.options[sourceIndex].selected = tempSelect;
}

/*
 * Selects all options in the list
 * param: selectId (select element id) - list
 */   
function SelectAllOptions(selectId) {
	var selectObj = document.getElementById(selectId);
	for (var i = 0; i < selectObj.options.length; i++) {
		selectObj.options[i].selected = true;
	}
}

/*
 * Selects all options in the list
 * param: selectId (select element id) - list
 */   
function SelectNoOptions(selectId) {
	var selectObj = document.getElementById(selectId);
	for (var i = 0; i < selectObj.options.length; i++) {
		selectObj.options[i].selected = false;
	}
}

/*
 * Bubblesorts the list by ASCII code
 * param: selectId (select element id) - list to sort
 */   
function SortOptions(selectId) {
	var selectObj = document.getElementById(selectId);
	var sorted = true;
	var option1 = "";
	var option2 = "";
	var len = 0;
	var charNum = 0;
	
	// Get longest string in the list
	for (var i = 0; i < selectObj.length; i++) {
		if (selectObj.options[i].text.length > len) {
			len = selectObj.options[i].text.length;
		}
	}
	
	// Get the character to analyse
	charNum = len - 1;
	
	// Keep looping until we have sorted all characters, from last to first
	while (charNum > -1) {
		sorted = true;
		
		// Keep looping whilst we are still sorting by this character
		while (sorted) {
			sorted = false;
			
			// Loop through all options, sorting by the current character
			for (var i = 0; i < selectObj.length - 1; i++) {
				option1 = selectObj.options[i].text.toLowerCase();
				option2 = selectObj.options[i+1].text.toLowerCase();
				
				// Check option text is long enough
				if ((option1.length >= charNum + 1) && (option2.length >= charNum + 1)) {
					// Sort by current character
					if (option1.charCodeAt(charNum) > option2.charCodeAt(charNum)) {
						// Sort necessary, so swap
						SwapOption(selectId, i, i+1);
						sorted = true;
					}
				} else if (option1.length >= charNum + 1) {
					// Option1 text longer than option2 text, so swap
					SwapOption(selectId, i, i+1);
					sorted = true;
				}
			}
		}
		
		// Decrease the character we're sorting by
		charNum--;
	}
}